@@ -1,9 +1,9 @@
 # SPDX-FileCopyrightText: 2022 James R. Barlow
 # SPDX-License-Identifier: MPL-2.0
-from __future__ import annotations
-
 """OCRmyPDF's multiprocessing/multithreading abstraction layer."""
 
+from __future__ import annotations
+
 import logging
 import logging.handlers
 import multiprocessing
@@ -12,25 +12,29 @@ import queue
 import signal
 import sys
 import threading
+from collections.abc import Callable, Iterable
 from concurrent.futures import ProcessPoolExecutor, ThreadPoolExecutor, as_completed
 from contextlib import suppress
-from typing import Callable, Iterable, Type, Union
+from typing import Union
 
-from tqdm import tqdm
+from rich.console import Console as RichConsole
 
 from ocrmypdf import Executor, hookimpl
-from ocrmypdf._logging import TqdmConsole
+from ocrmypdf._logging import RichLoggingHandler
+from ocrmypdf._progressbar import RichProgressBar
 from ocrmypdf.exceptions import InputFileError
 from ocrmypdf.helpers import remove_all_log_handlers
 
-FuturesExecutorClass = Union[Type[ThreadPoolExecutor], Type[ProcessPoolExecutor]]
-Queue = Union[multiprocessing.Queue, queue.Queue]
+FuturesExecutorClass = Union[  # noqa: UP007
+    type[ThreadPoolExecutor], type[ProcessPoolExecutor]
+]
+Queue = Union[multiprocessing.Queue, queue.Queue]  # noqa: UP007
 UserInit = Callable[[], None]
 WorkerInit = Callable[[Queue, UserInit, int], None]
 
 
 def log_listener(q: Queue):
-    """Listen to the worker processes and forward the messages to logging
+    """Listen to the worker processes and forward the messages to logging.
 
     For simplicity this is a thread rather than a process. Only one process
     should actually write to sys.stderr or whatever we're using, so if this is
@@ -39,7 +43,6 @@ def log_listener(q: Queue):
     See:
     https://docs.python.org/3/howto/logging-cookbook.html#logging-to-a-single-file-from-multiple-processes
     """
-
     while True:
         try:
             record = q.get()
@@ -55,12 +58,12 @@ def log_listener(q: Queue):
 
 
 def process_sigbus(*args):
+    """Handle SIGBUS signal at the worker level."""
     raise InputFileError("A worker process lost access to an input file")
 
 
 def process_init(q: Queue, user_init: UserInit, loglevel) -> None:
-    """Initialize a process pool worker"""
-
+    """Initialize a process pool worker."""
     # Ignore SIGINT (our parent process will kill us gracefully)
     signal.signal(signal.SIGINT, signal.SIG_IGN)
 
@@ -69,7 +72,7 @@ def process_init(q: Queue, user_init: UserInit, loglevel) -> None:
         # Windows and Cygwin do not have pthread_sigmask or SIGBUS
         signal.signal(signal.SIGBUS, process_sigbus)
 
-    # Remove any log handlers that belong to the parent process
+    # Remove any log handlers inherited from the parent process
     root = logging.getLogger()
     remove_all_log_handlers(root)
 
@@ -82,6 +85,7 @@ def process_init(q: Queue, user_init: UserInit, loglevel) -> None:
 
 
 def thread_init(q: Queue, user_init: UserInit, loglevel) -> None:
+    """Begin a thread pool worker."""
     del q  # unused but required argument
     del loglevel  # unused but required argument
     # As a thread, block SIGBUS so the main thread deals with it...
@@ -92,57 +96,74 @@ def thread_init(q: Queue, user_init: UserInit, loglevel) -> None:
     return
 
 
+def setup_executor(use_threads: bool) -> tuple[Queue, Executor, WorkerInit]:
+    if not use_threads:
+        # Some execution environments like AWS Lambda and Termux do not support
+        # semaphores. Check if semaphore support is available, and if not, fall back
+        # to using threads.
+        try:
+            # pylint: disable=import-outside-toplevel
+            from multiprocessing.synchronize import SemLock
+
+            del SemLock
+        except ImportError:
+            use_threads = True
+
+    if use_threads:
+        loq_queue = queue.Queue(-1)
+        executor_class = ThreadPoolExecutor
+        initializer = thread_init
+    else:
+        loq_queue = multiprocessing.Queue(-1)
+        executor_class = ProcessPoolExecutor
+        initializer = process_init
+
+    return loq_queue, executor_class, initializer
+
+
 class StandardExecutor(Executor):
     """Standard OCRmyPDF concurrent task executor."""
 
-    def _cancel_futures_kwargs(self):
-        """Shim older Pythons that do not have Executor.shutdown(...cancel_futures=).
-
-        Remove this code when support for Python 3.8 is dropped.
-        """
-        if sys.version_info[:2] < (3, 9):
-            return {}
-        return dict(cancel_futures=True)
-
     def _execute(
         self,
         *,
         use_threads: bool,
         max_workers: int,
-        tqdm_kwargs: dict,
+        progress_kwargs: dict,
         worker_initializer: Callable,
         task: Callable,
         task_arguments: Iterable,
         task_finished: Callable,
     ):
-        if use_threads:
-            log_queue: Queue = queue.Queue(-1)
-            executor_class: FuturesExecutorClass = ThreadPoolExecutor
-            initializer: WorkerInit = thread_init
-        else:
-            log_queue = multiprocessing.Queue(-1)
-            executor_class = ProcessPoolExecutor
-            initializer = process_init
+        log_queue, executor_class, initializer = setup_executor(use_threads)
 
         # Regardless of whether we use_threads for worker processes, the log_listener
         # must be a thread. Make sure we create the listener after the worker pool,
         # so that it does not get forked into the workers.
+        # If use_threads is False, we are currently guilty of creating a thread before
+        # forking on Linux, which is not recommended. However, we take a big
+        # performance hit in pdfinfo if we can't fork. Long term solution is to
+        # replace most of this with an asyncio implementation, and probably to
+        # migrate some of pdfinfo into C++ or Rust.
         listener = threading.Thread(target=log_listener, args=(log_queue,))
         listener.start()
 
-        with self.pbar_class(**tqdm_kwargs) as pbar, executor_class(
-            max_workers=max_workers,
-            initializer=initializer,
-            initargs=(log_queue, worker_initializer, logging.getLogger("").level),
-        ) as executor:
-            futures = [executor.submit(task, args) for args in task_arguments]
+        with (
+            self.pbar_class(**progress_kwargs) as pbar,
+            executor_class(
+                max_workers=max_workers,
+                initializer=initializer,
+                initargs=(log_queue, worker_initializer, logging.getLogger("").level),
+            ) as executor,
+        ):
+            futures = [executor.submit(task, *args) for args in task_arguments]
             try:
                 for future in as_completed(futures):
                     result = future.result()
                     task_finished(result, pbar)
             except KeyboardInterrupt:
                 # Terminate pool so we exit instantly
-                executor.shutdown(wait=False, **self._cancel_futures_kwargs())
+                executor.shutdown(wait=False, cancel_futures=True)
                 raise
             except Exception:
                 if not os.environ.get("PYTEST_CURRENT_TEST", ""):
@@ -151,7 +172,7 @@ class StandardExecutor(Executor):
                     # results will be discard. But if the condition above is True,
                     # then we are running in pytest, and we want everything to exit
                     # as cleanly as possible so that we get good error messages.
-                    executor.shutdown(wait=False, **self._cancel_futures_kwargs())
+                    executor.shutdown(wait=False, cancel_futures=True)
                 raise
             finally:
                 # Terminate log listener
@@ -164,14 +185,24 @@ class StandardExecutor(Executor):
 
 @hookimpl
 def get_executor(progressbar_class):
+    """Return the default executor."""
     return StandardExecutor(pbar_class=progressbar_class)
 
 
+RICH_CONSOLE = RichConsole(stderr=True)
+
+
 @hookimpl
 def get_progressbar_class():
-    return tqdm
+    """Return the default progress bar class."""
+
+    def partial_RichProgressBar(*args, **kwargs):
+        return RichProgressBar(*args, **kwargs, console=RICH_CONSOLE)
+
+    return partial_RichProgressBar
 
 
 @hookimpl
 def get_logging_console():
-    return logging.StreamHandler(stream=TqdmConsole(sys.stderr))
+    """Return the default logging console handler."""
+    return RichLoggingHandler(console=RICH_CONSOLE)
