@@ -6,6 +6,8 @@ from __future__ import annotations
 
 import logging
 
+from packaging.version import Version
+
 from ocrmypdf import hookimpl
 from ocrmypdf._exec import ghostscript
 from ocrmypdf.exceptions import MissingDependencyError
@@ -15,7 +17,7 @@ log = logging.getLogger(__name__)
 
 # Currently all blacklisted versions are lower than 9.55, so none need to
 # be added here. If a future version is blacklisted, add it here.
-BLACKLISTED_GS_VERSIONS: frozenset[str] = frozenset()
+BLACKLISTED_GS_VERSIONS: frozenset[Version] = frozenset()
 
 
 @hookimpl
@@ -52,14 +54,24 @@ def check_options(options):
         program='gs',
         package='ghostscript',
         version_checker=ghostscript.version,
-        need_version='9.55',  # Ubuntu 22.04's version
+        need_version='9.54',  # RHEL 9's version; Ubuntu 22.04 has 9.55
     )
     gs_version = ghostscript.version()
     if gs_version in BLACKLISTED_GS_VERSIONS:
         raise MissingDependencyError(
             f"Ghostscript {gs_version} contains serious regressions and is not "
-            "supported. Please upgrade to a newer version, or downgrade to the "
-            "previous version."
+            "supported. Please upgrade to a newer version."
+        )
+    if Version('10.0.0') <= gs_version < Version('10.02.1') and (
+        options.skip_text or options.redo_ocr
+    ):
+        raise MissingDependencyError(
+            f"Ghostscript 10.0.0 through 10.02.0 (your version: {gs_version}) "
+            "contain serious regressions that corrupt PDFs with existing text, "
+            "such as those processed using --skip-text or --redo-ocr. "
+            "Please upgrade to a "
+            "newer version, or use --output-type pdf to avoid Ghostscript, or "
+            "use --force-ocr to discard existing text."
         )
 
     if options.output_type == 'pdfa':
@@ -117,7 +129,7 @@ def generate_pdfa(
 ):
     """Generate a PDF/A from the list of PDF pages and PDF/A metadata."""
     ghostscript.generate_pdfa(
-        pdf_pages=[*pdf_pages, pdfmark],
+        pdf_pages=[pdfmark, *pdf_pages],
         output_file=output_file,
         compression=context.options.pdfa_image_compression,
         color_conversion_strategy=context.options.color_conversion_strategy,
